@@ -1,7 +1,7 @@
 /*
  * Copyright 2008-2009 Katholieke Universiteit Leuven
  *
- * Use of this software is governed by the GNU LGPLv2.1 license
+ * Use of this software is governed by the MIT license
  *
  * Written by Sven Verdoolaege, K.U.Leuven, Departement
  * Computerwetenschappen, Celestijnenlaan 200A, B-3001 Leuven, Belgium
@@ -13,8 +13,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-#include <isl/int.h>
-#include <isl/blk.h>
 #include <isl/arg.h>
 #include <isl/hash.h>
 #include <isl/config.h>
@@ -28,6 +26,9 @@
 #ifndef __isl_keep
 #define __isl_keep
 #endif
+#ifndef __isl_null
+#define __isl_null
+#endif
 #ifndef __isl_export
 #define __isl_export
 #endif
@@ -78,9 +79,11 @@ struct isl_stats {
 enum isl_error {
 	isl_error_none = 0,
 	isl_error_abort,
+	isl_error_alloc,
 	isl_error_unknown,
 	isl_error_internal,
 	isl_error_invalid,
+	isl_error_quota,
 	isl_error_unsupported
 };
 struct isl_ctx;
@@ -98,24 +101,21 @@ typedef struct isl_ctx isl_ctx;
 #define ISL_F_CLR(p, f)     ISL_FL_CLR((p)->flags, f)
 #define ISL_F_ISSET(p, f)   ISL_FL_ISSET((p)->flags, f)
 
-/* isl_check_ctx() checks at compile time if 'ctx' is of type 'isl_ctx *' and
- * returns the value of 'expr'. It is used to ensure, that always an isl_ctx is
- * passed to the following macros, even if they currently do not use it.
- */
-#define isl_check_ctx(ctx, expr)	((ctx != (isl_ctx *) 0) ? expr : expr)
+void *isl_malloc_or_die(isl_ctx *ctx, size_t size);
+void *isl_calloc_or_die(isl_ctx *ctx, size_t nmemb, size_t size);
+void *isl_realloc_or_die(isl_ctx *ctx, void *ptr, size_t size);
 
-#define isl_alloc(ctx,type,size)	((type *)isl_check_ctx(ctx,\
-							malloc(size)))
-#define isl_calloc(ctx,type,size)	((type *)isl_check_ctx(ctx,\
-							calloc(1, size)))
-#define isl_realloc(ctx,ptr,type,size)	((type *)isl_check_ctx(ctx,\
-							realloc(ptr,size)))
+#define isl_alloc(ctx,type,size)	((type *)isl_malloc_or_die(ctx, size))
+#define isl_calloc(ctx,type,size)	((type *)isl_calloc_or_die(ctx,\
+								    1, size))
+#define isl_realloc(ctx,ptr,type,size)	((type *)isl_realloc_or_die(ctx,\
+								    ptr, size))
 #define isl_alloc_type(ctx,type)	isl_alloc(ctx,type,sizeof(type))
 #define isl_calloc_type(ctx,type)	isl_calloc(ctx,type,sizeof(type))
 #define isl_realloc_type(ctx,ptr,type)	isl_realloc(ctx,ptr,type,sizeof(type))
 #define isl_alloc_array(ctx,type,n)	isl_alloc(ctx,type,(n)*sizeof(type))
-#define isl_calloc_array(ctx,type,n)	((type *)isl_check_ctx(ctx,\
-						calloc(n, sizeof(type))))
+#define isl_calloc_array(ctx,type,n)	((type *)isl_calloc_or_die(ctx,\
+							    n, sizeof(type)))
 #define isl_realloc_array(ctx,ptr,type,n) \
 				    isl_realloc(ctx,ptr,type,(n)*sizeof(type))
 
@@ -156,6 +156,10 @@ void isl_ctx_abort(isl_ctx *ctx);
 void isl_ctx_resume(isl_ctx *ctx);
 int isl_ctx_aborted(isl_ctx *ctx);
 
+void isl_ctx_set_max_operations(isl_ctx *ctx, unsigned long max_operations);
+unsigned long isl_ctx_get_max_operations(isl_ctx *ctx);
+void isl_ctx_reset_operations(isl_ctx *ctx);
+
 #define ISL_ARG_CTX_DECL(prefix,st,args)				\
 st *isl_ctx_peek_ ## prefix(isl_ctx *ctx);
 
@@ -190,6 +194,36 @@ int prefix ## _set_ ## field(isl_ctx *ctx, int val)			\
 	return 0;							\
 }
 
+#define ISL_CTX_GET_STR_DEF(prefix,st,args,field)			\
+const char *prefix ## _get_ ## field(isl_ctx *ctx)			\
+{									\
+	st *options;							\
+	options = isl_ctx_peek_ ## prefix(ctx);				\
+	if (!options)							\
+		isl_die(ctx, isl_error_invalid,				\
+			"isl_ctx does not reference " #prefix,		\
+			return NULL);					\
+	return options->field;						\
+}
+
+#define ISL_CTX_SET_STR_DEF(prefix,st,args,field)			\
+int prefix ## _set_ ## field(isl_ctx *ctx, const char *val)		\
+{									\
+	st *options;							\
+	options = isl_ctx_peek_ ## prefix(ctx);				\
+	if (!options)							\
+		isl_die(ctx, isl_error_invalid,				\
+			"isl_ctx does not reference " #prefix,		\
+			return -1);					\
+	if (!val)							\
+		return -1;						\
+	free(options->field);						\
+	options->field = strdup(val);					\
+	if (!options->field)						\
+		return -1;						\
+	return 0;							\
+}
+
 #define ISL_CTX_GET_BOOL_DEF(prefix,st,args,field)			\
 	ISL_CTX_GET_INT_DEF(prefix,st,args,field)
 
