@@ -35,6 +35,7 @@
 
 #include <assert.h>
 #include <iostream>
+#include <llvm/ADT/OwningPtr.h>
 #include <llvm/Support/raw_ostream.h>
 #include <llvm/Support/CommandLine.h>
 #include <llvm/Support/Host.h>
@@ -76,7 +77,8 @@ static llvm::cl::list<string> Includes("I",
 			llvm::cl::desc("Header search path"),
 			llvm::cl::value_desc("path"), llvm::cl::Prefix);
 
-static const char *ResourceDir = CLANG_PREFIX"/lib/clang/"CLANG_VERSION_STRING;
+static const char *ResourceDir =
+	CLANG_PREFIX "/lib/clang/" CLANG_VERSION_STRING;
 
 /* Does decl have an attribute of the following form?
  *
@@ -254,6 +256,22 @@ static void create_diagnostics(CompilerInstance *Clang)
 
 #endif
 
+#ifdef CREATEPREPROCESSOR_TAKES_TUKIND
+
+static void create_preprocessor(CompilerInstance *Clang)
+{
+	Clang->createPreprocessor(TU_Complete);
+}
+
+#else
+
+static void create_preprocessor(CompilerInstance *Clang)
+{
+	Clang->createPreprocessor();
+}
+
+#endif
+
 #ifdef ADDPATH_TAKES_4_ARGUMENTS
 
 void add_path(HeaderSearchOptions &HSO, string Path)
@@ -303,7 +321,7 @@ int main(int argc, char *argv[])
 	PO.addMacroDef("__isl_constructor=__attribute__((annotate(\"isl_constructor\"))) __attribute__((annotate(\"isl_export\")))");
 	PO.addMacroDef("__isl_subclass(super)=__attribute__((annotate(\"isl_subclass(\" #super \")\"))) __attribute__((annotate(\"isl_export\")))");
 
-	Clang->createPreprocessor();
+	create_preprocessor(Clang);
 	Preprocessor &PP = Clang->getPreprocessor();
 
 	PP.getBuiltinInfo().InitializeBuiltins(PP.getIdentifierTable(), LO);
