@@ -1,5 +1,13 @@
+/*
+ * Copyright 2012-2013 Ecole Normale Superieure
+ *
+ * Use of this software is governed by the MIT license
+ *
+ * Written by Sven Verdoolaege,
+ * Ecole Normale Superieure, 45 rue d’Ulm, 75230 Paris, France
+ */
+
 #include <isl_ast_private.h>
-#include <isl/val_int.h>
 
 #undef BASE
 #define BASE ast_expr
@@ -76,7 +84,8 @@ __isl_give isl_ast_print_options *isl_ast_print_options_copy(
 	return options;
 }
 
-void *isl_ast_print_options_free(__isl_take isl_ast_print_options *options)
+__isl_null isl_ast_print_options *isl_ast_print_options_free(
+	__isl_take isl_ast_print_options *options)
 {
 	if (!options)
 		return NULL;
@@ -189,7 +198,7 @@ __isl_give isl_ast_expr *isl_ast_expr_cow(__isl_take isl_ast_expr *expr)
 	return isl_ast_expr_dup(expr);
 }
 
-void *isl_ast_expr_free(__isl_take isl_ast_expr *expr)
+__isl_null isl_ast_expr *isl_ast_expr_free(__isl_take isl_ast_expr *expr)
 {
 	int i;
 
@@ -209,8 +218,9 @@ void *isl_ast_expr_free(__isl_take isl_ast_expr *expr)
 		isl_id_free(expr->u.id);
 		break;
 	case isl_ast_expr_op:
-		for (i = 0; i < expr->u.op.n_arg; ++i)
-			isl_ast_expr_free(expr->u.op.args[i]);
+		if (expr->u.op.args)
+			for (i = 0; i < expr->u.op.n_arg; ++i)
+				isl_ast_expr_free(expr->u.op.args[i]);
 		free(expr->u.op.args);
 		break;
 	case isl_ast_expr_error:
@@ -231,16 +241,6 @@ enum isl_ast_expr_type isl_ast_expr_get_type(__isl_keep isl_ast_expr *expr)
 	return expr ? expr->type : isl_ast_expr_error;
 }
 
-int isl_ast_expr_get_int(__isl_keep isl_ast_expr *expr, isl_int *v)
-{
-	if (!expr)
-		return -1;
-	if (expr->type != isl_ast_expr_int)
-		isl_die(isl_ast_expr_get_ctx(expr), isl_error_invalid,
-			"expression not an int", return -1);
-	return isl_val_get_num_isl_int(expr->u.v, v);
-}
-
 /* Return the integer value represented by "expr".
  */
 __isl_give isl_val *isl_ast_expr_get_val(__isl_keep isl_ast_expr *expr)
@@ -323,6 +323,44 @@ error:
 	return isl_ast_expr_free(expr);
 }
 
+/* Is "expr1" equal to "expr2"?
+ */
+int isl_ast_expr_is_equal(__isl_keep isl_ast_expr *expr1,
+	__isl_keep isl_ast_expr *expr2)
+{
+	int i;
+
+	if (!expr1 || !expr2)
+		return -1;
+
+	if (expr1 == expr2)
+		return 1;
+	if (expr1->type != expr2->type)
+		return 0;
+	switch (expr1->type) {
+	case isl_ast_expr_int:
+		return isl_val_eq(expr1->u.v, expr2->u.v);
+	case isl_ast_expr_id:
+		return expr1->u.id == expr2->u.id;
+	case isl_ast_expr_op:
+		if (expr1->u.op.op != expr2->u.op.op)
+			return 0;
+		if (expr1->u.op.n_arg != expr2->u.op.n_arg)
+			return 0;
+		for (i = 0; i < expr1->u.op.n_arg; ++i) {
+			int equal;
+			equal = isl_ast_expr_is_equal(expr1->u.op.args[i],
+							expr2->u.op.args[i]);
+				return 0;
+			if (equal < 0 || !equal)
+				return equal;
+		}
+		return 1;
+	case isl_ast_expr_error:
+		return -1;
+	}
+}
+
 /* Create a new operation expression of operation type "op",
  * with "n_arg" as yet unspecified arguments.
  */
@@ -362,7 +400,7 @@ __isl_give isl_ast_expr *isl_ast_expr_from_id(__isl_take isl_id *id)
 	ctx = isl_id_get_ctx(id);
 	expr = isl_calloc_type(ctx, isl_ast_expr);
 	if (!expr)
-		return isl_id_free(id);
+		goto error;
 
 	expr->ctx = ctx;
 	isl_ctx_ref(ctx);
@@ -371,6 +409,9 @@ __isl_give isl_ast_expr *isl_ast_expr_from_id(__isl_take isl_id *id)
 	expr->u.id = id;
 
 	return expr;
+error:
+	isl_id_free(id);
+	return NULL;
 }
 
 /* Create a new integer expression representing "i".
@@ -405,12 +446,12 @@ __isl_give isl_ast_expr *isl_ast_expr_from_val(__isl_take isl_val *v)
 		return NULL;
 	if (!isl_val_is_int(v))
 		isl_die(isl_val_get_ctx(v), isl_error_invalid,
-			"expecting integer value", return isl_val_free(v));
+			"expecting integer value", goto error);
 
 	ctx = isl_val_get_ctx(v);
 	expr = isl_calloc_type(ctx, isl_ast_expr);
 	if (!expr)
-		return isl_val_free(v);
+		goto error;
 
 	expr->ctx = ctx;
 	isl_ctx_ref(ctx);
@@ -419,6 +460,9 @@ __isl_give isl_ast_expr *isl_ast_expr_from_val(__isl_take isl_val *v)
 	expr->u.v = v;
 
 	return expr;
+error:
+	isl_val_free(v);
+	return NULL;
 }
 
 /* Create an expression representing the negation of "arg".
@@ -519,6 +563,99 @@ __isl_give isl_ast_expr *isl_ast_expr_or(__isl_take isl_ast_expr *expr1,
 	return isl_ast_expr_alloc_binary(isl_ast_op_or, expr1, expr2);
 }
 
+/* Create an expression representing an access to "array" with index
+ * expressions "indices".
+ */
+__isl_give isl_ast_expr *isl_ast_expr_access(__isl_take isl_ast_expr *array,
+	__isl_take isl_ast_expr_list *indices)
+{
+	int i, n;
+	isl_ctx *ctx;
+	isl_ast_expr *access = NULL;
+
+	if (!array || !indices)
+		goto error;
+
+	ctx = isl_ast_expr_get_ctx(array);
+	n = isl_ast_expr_list_n_ast_expr(indices);
+	access = isl_ast_expr_alloc_op(ctx, isl_ast_op_access, 1 + n);
+	if (!access)
+		goto error;
+	for (i = 0; i < n; ++i) {
+		isl_ast_expr *index;
+		index = isl_ast_expr_list_get_ast_expr(indices, i);
+		access->u.op.args[1 + i] = index;
+		if (!index)
+			goto error;
+	}
+	access->u.op.args[0] = array;
+
+	isl_ast_expr_list_free(indices);
+	return access;
+error:
+	isl_ast_expr_free(array);
+	isl_ast_expr_list_free(indices);
+	isl_ast_expr_free(access);
+	return NULL;
+}
+
+/* For each subexpression of "expr" of type isl_ast_expr_id,
+ * if it appears in "id2expr", then replace it by the corresponding
+ * expression.
+ */
+__isl_give isl_ast_expr *isl_ast_expr_substitute_ids(
+	__isl_take isl_ast_expr *expr, __isl_take isl_id_to_ast_expr *id2expr)
+{
+	int i;
+	isl_id *id;
+
+	if (!expr || !id2expr)
+		goto error;
+
+	switch (expr->type) {
+	case isl_ast_expr_int:
+		break;
+	case isl_ast_expr_id:
+		if (!isl_id_to_ast_expr_has(id2expr, expr->u.id))
+			break;
+		id = isl_id_copy(expr->u.id);
+		isl_ast_expr_free(expr);
+		expr = isl_id_to_ast_expr_get(id2expr, id);
+		break;
+	case isl_ast_expr_op:
+		for (i = 0; i < expr->u.op.n_arg; ++i) {
+			isl_ast_expr *arg;
+			arg = isl_ast_expr_copy(expr->u.op.args[i]);
+			arg = isl_ast_expr_substitute_ids(arg,
+					    isl_id_to_ast_expr_copy(id2expr));
+			if (arg == expr->u.op.args[i]) {
+				isl_ast_expr_free(arg);
+				continue;
+			}
+			if (!arg)
+				expr = isl_ast_expr_free(expr);
+			expr = isl_ast_expr_cow(expr);
+			if (!expr) {
+				isl_ast_expr_free(arg);
+				break;
+			}
+			isl_ast_expr_free(expr->u.op.args[i]);
+			expr->u.op.args[i] = arg;
+		}
+		break;
+	case isl_ast_expr_error:
+		expr = isl_ast_expr_free(expr);
+		break;
+	}
+
+	isl_id_to_ast_expr_free(id2expr);
+	return expr;
+error:
+	isl_ast_expr_free(expr);
+	isl_id_to_ast_expr_free(id2expr);
+	return NULL;
+}
+
 isl_ctx *isl_ast_node_get_ctx(__isl_keep isl_ast_node *node)
 {
 	return node ? node->ctx : NULL;
@@ -583,13 +720,16 @@ __isl_give isl_ast_node *isl_ast_node_alloc_for(__isl_take isl_id *id)
 	ctx = isl_id_get_ctx(id);
 	node = isl_ast_node_alloc(ctx, isl_ast_node_for);
 	if (!node)
-		return NULL;
+		goto error;
 
 	node->u.f.iterator = isl_ast_expr_from_id(id);
 	if (!node->u.f.iterator)
 		return isl_ast_node_free(node);
 
 	return node;
+error:
+	isl_id_free(id);
+	return NULL;
 }
 
 /* Create a user node evaluating "expr".
@@ -724,7 +864,7 @@ __isl_give isl_ast_node *isl_ast_node_cow(__isl_take isl_ast_node *node)
 	return isl_ast_node_dup(node);
 }
 
-void *isl_ast_node_free(__isl_take isl_ast_node *node)
+__isl_null isl_ast_node *isl_ast_node_free(__isl_take isl_ast_node *node)
 {
 	if (!node)
 		return NULL;
@@ -1009,7 +1149,8 @@ static char *op_str[] = {
 	[isl_ast_op_le] = "<=",
 	[isl_ast_op_ge] = ">=",
 	[isl_ast_op_lt] = "<",
-	[isl_ast_op_gt] = ">"
+	[isl_ast_op_gt] = ">",
+	[isl_ast_op_member] = "."
 };
 
 /* Precedence in C of the various operators.
@@ -1038,7 +1179,9 @@ static int op_prec[] = {
 	[isl_ast_op_ge] = 8,
 	[isl_ast_op_lt] = 8,
 	[isl_ast_op_gt] = 8,
-	[isl_ast_op_call] = 2
+	[isl_ast_op_call] = 2,
+	[isl_ast_op_access] = 2,
+	[isl_ast_op_member] = 2
 };
 
 /* Is the operator left-to-right associative?
@@ -1065,7 +1208,9 @@ static int op_left[] = {
 	[isl_ast_op_ge] = 1,
 	[isl_ast_op_lt] = 1,
 	[isl_ast_op_gt] = 1,
-	[isl_ast_op_call] = 1
+	[isl_ast_op_call] = 1,
+	[isl_ast_op_access] = 1,
+	[isl_ast_op_member] = 1
 };
 
 static int is_and(enum isl_ast_op_type op)
@@ -1188,6 +1333,25 @@ static __isl_give isl_printer *print_call(__isl_take isl_printer *p,
 	return p;
 }
 
+/* Print an array access "expr".
+ *
+ * The first argument represents the array being accessed.
+ */
+static __isl_give isl_printer *print_access(__isl_take isl_printer *p,
+	__isl_keep isl_ast_expr *expr)
+{
+	int i = 0;
+
+	p = isl_printer_print_ast_expr(p, expr->u.op.args[0]);
+	for (i = 1; i < expr->u.op.n_arg; ++i) {
+		p = isl_printer_print_str(p, "[");
+		p = isl_printer_print_ast_expr(p, expr->u.op.args[i]);
+		p = isl_printer_print_str(p, "]");
+	}
+
+	return p;
+}
+
 /* Print "expr" to "p".
  *
  * If we are printing in isl format, then we also print an indication
@@ -1207,6 +1371,10 @@ __isl_give isl_printer *isl_printer_print_ast_expr(__isl_take isl_printer *p,
 			p = print_call(p, expr);
 			break;
 		}
+		if (expr->u.op.op == isl_ast_op_access) {
+			p = print_access(p, expr);
+			break;
+		}
 		if (expr->u.op.n_arg == 1) {
 			p = isl_printer_print_str(p, op_str[expr->u.op.op]);
 			p = print_sub_expr(p, expr->u.op.op,
@@ -1240,9 +1408,11 @@ __isl_give isl_printer *isl_printer_print_ast_expr(__isl_take isl_printer *p,
 				"operation should have two arguments",
 				goto error);
 		p = print_sub_expr(p, expr->u.op.op, expr->u.op.args[0], 1);
-		p = isl_printer_print_str(p, " ");
+		if (expr->u.op.op != isl_ast_op_member)
+			p = isl_printer_print_str(p, " ");
 		p = isl_printer_print_str(p, op_str[expr->u.op.op]);
-		p = isl_printer_print_str(p, " ");
+		if (expr->u.op.op != isl_ast_op_member)
+			p = isl_printer_print_str(p, " ");
 		p = print_sub_expr(p, expr->u.op.op, expr->u.op.args[1], 0);
 		break;
 	case isl_ast_expr_id:
@@ -1320,6 +1490,8 @@ static __isl_give isl_printer *print_ast_node_isl(__isl_take isl_printer *p,
  * Also if the node is a degenerate for then we will print it as
  * an assignment followed by the body of the for loop, so we need a block
  * as well.
+ * If the node is an if node with an else, then we print a block
+ * to avoid spurious dangling else warnings emitted by some compilers.
  */
 static int need_block(__isl_keep isl_ast_node *node)
 {
@@ -1327,6 +1499,8 @@ static int need_block(__isl_keep isl_ast_node *node)
 		return 1;
 	if (node->type == isl_ast_node_for && node->u.f.degenerate)
 		return 1;
+	if (node->type == isl_ast_node_if && node->u.i.else_node)
+		return 1;
 	return 0;
 }
 
