@@ -13,10 +13,12 @@
 #include <isl_map_private.h>
 #include <isl_constraint_private.h>
 #include <isl_space_private.h>
-#include <isl/seq.h>
+#include <isl_seq.h>
 #include <isl_aff_private.h>
 #include <isl_local_space_private.h>
 #include <isl_val_private.h>
+#include <isl_vec_private.h>
+#include <isl/deprecated/constraint_int.h>
 
 #undef BASE
 #define BASE constraint
@@ -172,7 +174,7 @@ struct isl_constraint *isl_constraint_copy(struct isl_constraint *constraint)
 	return constraint;
 }
 
-void *isl_constraint_free(struct isl_constraint *c)
+__isl_null isl_constraint *isl_constraint_free(__isl_take isl_constraint *c)
 {
 	if (!c)
 		return NULL;
@@ -548,7 +550,7 @@ __isl_give isl_constraint *isl_constraint_set_coefficient(
  */
 __isl_give isl_constraint *isl_constraint_set_coefficient_val(
 	__isl_take isl_constraint *constraint,
-	enum isl_dim_type type, int pos, isl_val *v)
+	enum isl_dim_type type, int pos, __isl_take isl_val *v)
 {
 	constraint = isl_constraint_cow(constraint);
 	if (!constraint || !v)
@@ -750,9 +752,11 @@ struct isl_basic_set *isl_basic_set_from_constraint(
 
 	if (isl_constraint_dim(constraint, isl_dim_in) != 0)
 		isl_die(isl_constraint_get_ctx(constraint), isl_error_invalid,
-			"not a set constraint",
-			return isl_constraint_free(constraint));
+			"not a set constraint", goto error);
 	return (isl_basic_set *)isl_basic_map_from_constraint(constraint);
+error:
+	isl_constraint_free(constraint);
+	return NULL;
 }
 
 int isl_basic_map_has_defining_equality(
