@@ -14,10 +14,13 @@
 #include <isl_polynomial_private.h>
 #include <isl_point_private.h>
 #include <isl_space_private.h>
-#include <isl/lp.h>
-#include <isl/seq.h>
+#include <isl_lp_private.h>
+#include <isl_seq.h>
 #include <isl_mat_private.h>
+#include <isl_val_private.h>
+#include <isl_vec_private.h>
 #include <isl_config.h>
+#include <isl/deprecated/polynomial_int.h>
 
 enum isl_fold isl_fold_type_negate(enum isl_fold type)
 {
@@ -650,6 +653,8 @@ __isl_give isl_qpolynomial_fold *isl_qpolynomial_fold_gist_params(
 #define PARTS pw_qpolynomial_fold
 #define ALIGN_DOMAIN
 
+#define NO_SUB
+
 #include <isl_union_templ.c>
 
 __isl_give isl_qpolynomial_fold *isl_qpolynomial_fold_empty(enum isl_fold type,
@@ -999,39 +1004,41 @@ int isl_qpolynomial_fold_plain_is_equal(__isl_keep isl_qpolynomial_fold *fold1,
 	return 1;
 }
 
-__isl_give isl_qpolynomial *isl_qpolynomial_fold_eval(
+__isl_give isl_val *isl_qpolynomial_fold_eval(
 	__isl_take isl_qpolynomial_fold *fold, __isl_take isl_point *pnt)
 {
-	isl_qpolynomial *qp;
+	isl_ctx *ctx;
+	isl_val *v;
 
 	if (!fold || !pnt)
 		goto error;
+	ctx = isl_point_get_ctx(pnt);
 	isl_assert(pnt->dim->ctx, isl_space_is_equal(pnt->dim, fold->dim), goto error);
 	isl_assert(pnt->dim->ctx,
 		fold->type == isl_fold_max || fold->type == isl_fold_min,
 		goto error);
 
 	if (fold->n == 0)
-		qp = isl_qpolynomial_zero_on_domain(isl_space_copy(fold->dim));
+		v = isl_val_zero(ctx);
 	else {
 		int i;
-		qp = isl_qpolynomial_eval(isl_qpolynomial_copy(fold->qp[0]),
+		v = isl_qpolynomial_eval(isl_qpolynomial_copy(fold->qp[0]),
 						isl_point_copy(pnt));
 		for (i = 1; i < fold->n; ++i) {
-			isl_qpolynomial *qp_i;
-			qp_i = isl_qpolynomial_eval(
+			isl_val *v_i;
+			v_i = isl_qpolynomial_eval(
 					    isl_qpolynomial_copy(fold->qp[i]),
 					    isl_point_copy(pnt));
 			if (fold->type == isl_fold_max)
-				qp = isl_qpolynomial_max_cst(qp, qp_i);
+				v = isl_val_max(v, v_i);
 			else
-				qp = isl_qpolynomial_min_cst(qp, qp_i);
+				v = isl_val_min(v, v_i);
 		}
 	}
 	isl_qpolynomial_fold_free(fold);
 	isl_point_free(pnt);
 
-	return qp;
+	return v;
 error:
 	isl_qpolynomial_fold_free(fold);
 	isl_point_free(pnt);
@@ -1049,33 +1056,33 @@ size_t isl_pw_qpolynomial_fold_size(__isl_keep isl_pw_qpolynomial_fold *pwf)
 	return n;
 }
 
-__isl_give isl_qpolynomial *isl_qpolynomial_fold_opt_on_domain(
+__isl_give isl_val *isl_qpolynomial_fold_opt_on_domain(
 	__isl_take isl_qpolynomial_fold *fold, __isl_take isl_set *set, int max)
 {
 	int i;
-	isl_qpolynomial *opt;
+	isl_val *opt;
 
 	if (!set || !fold)
 		goto error;
 
 	if (fold->n == 0) {
-		isl_space *dim = isl_space_copy(fold->dim);
+		opt = isl_val_zero(isl_set_get_ctx(set));
 		isl_set_free(set);
 		isl_qpolynomial_fold_free(fold);
-		return isl_qpolynomial_zero_on_domain(dim);
+		return opt;
 	}
 
 	opt = isl_qpolynomial_opt_on_domain(isl_qpolynomial_copy(fold->qp[0]),
 						isl_set_copy(set), max);
 	for (i = 1; i < fold->n; ++i) {
-		isl_qpolynomial *opt_i;
+		isl_val *opt_i;
 		opt_i = isl_qpolynomial_opt_on_domain(
 				isl_qpolynomial_copy(fold->qp[i]),
 				isl_set_copy(set), max);
 		if (max)
-			opt = isl_qpolynomial_max_cst(opt, opt_i);
+			opt = isl_val_max(opt, opt_i);
 		else
-			opt = isl_qpolynomial_min_cst(opt, opt_i);
+			opt = isl_val_min(opt, opt_i);
 	}
 
 	isl_set_free(set);
@@ -1628,3 +1635,50 @@ __isl_give isl_qpolynomial_fold *isl_qpolynomial_fold_scale(
 {
 	return isl_qpolynomial_fold_mul_isl_int(fold, v);
 }
+
+/* Multiply "fold" by "v".
+ */
+__isl_give isl_qpolynomial_fold *isl_qpolynomial_fold_scale_val(
+	__isl_take isl_qpolynomial_fold *fold, __isl_take isl_val *v)
+{
+	int i;
+
+	if (!fold || !v)
+		goto error;
+
+	if (isl_val_is_one(v)) {
+		isl_val_free(v);
+		return fold;
+	}
+	if (isl_val_is_zero(v)) {
+		isl_qpolynomial_fold *zero;
+		isl_space *space = isl_qpolynomial_fold_get_domain_space(fold);
+		zero = isl_qpolynomial_fold_empty(fold->type, space);
+		isl_qpolynomial_fold_free(fold);
+		isl_val_free(v);
+		return zero;
+	}
+	if (!isl_val_is_rat(v))
+		isl_die(isl_qpolynomial_fold_get_ctx(fold), isl_error_invalid,
+			"expecting rational factor", goto error);
+
+	fold = isl_qpolynomial_fold_cow(fold);
+	if (!fold)
+		goto error;
+
+	if (isl_val_is_neg(v))
+		fold->type = isl_fold_type_negate(fold->type);
+	for (i = 0; i < fold->n; ++i) {
+		fold->qp[i] = isl_qpolynomial_scale_val(fold->qp[i],
+							isl_val_copy(v));
+		if (!fold->qp[i])
+			goto error;
+	}
+
+	isl_val_free(v);
+	return fold;
+error:
+	isl_val_free(v);
+	isl_qpolynomial_fold_free(fold);
+	return NULL;
+}
