@@ -8,11 +8,12 @@
  */
 
 #include <isl_map_private.h>
-#include <isl/seq.h>
+#include <isl_seq.h>
 #include <isl/set.h>
 #include <isl/map.h>
 #include "isl_tab.h"
 #include <isl_point_private.h>
+#include <isl_vec_private.h>
 
 /* Expand the constraint "c" into "v".  The initial "dim" dimensions
  * are the same, but "v" may have more divs than "c" and the divs of "c"
@@ -228,6 +229,8 @@ static int n_non_redundant(isl_ctx *ctx, struct isl_tab *tab,
 	if (isl_tab_detect_redundant(tab) < 0)
 		return -1;
 
+	if (n_test == 0)
+		return 0;
 	if (!*index)
 		*index = isl_alloc_array(ctx, int, n_test);
 	if (!*index)
