@@ -10,10 +10,12 @@
 #include <isl_ctx_private.h>
 #include <isl_map_private.h>
 #include <isl/space.h>
-#include <isl/seq.h>
+#include <isl_seq.h>
 #include <isl_mat_private.h>
+#include <isl_vec_private.h>
 #include <isl_space_private.h>
 #include <isl_val_private.h>
+#include <isl/deprecated/mat_int.h>
 
 isl_ctx *isl_mat_get_ctx(__isl_keep isl_mat *mat)
 {
@@ -205,7 +207,7 @@ struct isl_mat *isl_mat_cow(struct isl_mat *mat)
 	return mat2;
 }
 
-void *isl_mat_free(struct isl_mat *mat)
+__isl_null isl_mat *isl_mat_free(__isl_take isl_mat *mat)
 {
 	if (!mat)
 		return NULL;
@@ -530,7 +532,7 @@ static void oppose(struct isl_mat *M, struct isl_mat **U,
  * with U and Q unimodular matrices and H a matrix in column echelon form
  * such that on each echelon row the entries in the non-echelon column
  * are non-negative (if neg == 0) or non-positive (if neg == 1)
- * and stricly smaller (in absolute value) than the entries in the echelon
+ * and strictly smaller (in absolute value) than the entries in the echelon
  * column.
  * If U or Q are NULL, then these matrices are not computed.
  */
