@@ -52,7 +52,7 @@ static __isl_give UNION *FN(UNION,alloc)(__isl_take isl_space *dim, int size)
 
 	u = isl_calloc_type(dim->ctx, UNION);
 	if (!u)
-		return NULL;
+		goto error;
 
 	u->ref = 1;
 #ifdef HAS_TYPE
@@ -60,12 +60,11 @@ static __isl_give UNION *FN(UNION,alloc)(__isl_take isl_space *dim, int size)
 #endif
 	u->dim = dim;
 	if (isl_hash_table_init(dim->ctx, &u->table, size) < 0)
-		goto error;
+		return FN(UNION,free)(u);
 
 	return u;
 error:
 	isl_space_free(dim);
-	FN(UNION,free)(u);
 	return NULL;
 }
 
@@ -163,14 +162,14 @@ __isl_give UNION *FN(FN(UNION,add),PARTS)(__isl_take UNION *u,
 		return u;
 	}
 
+	u = FN(UNION,align_params)(u, FN(PART,get_space)(part));
+	part = FN(PART,align_params)(part, FN(UNION,get_space)(u));
+
 	u = FN(UNION,cow)(u);
 
 	if (!u)
 		goto error;
 
-	isl_assert(u->dim->ctx, isl_space_match(part->dim, isl_dim_param, u->dim,
-					      isl_dim_param), goto error);
-
 	hash = isl_space_get_hash(part->dim);
 	entry = isl_hash_table_find(u->dim->ctx, &u->table, hash,
 				    &has_dim, part->dim, 1);
@@ -244,7 +243,7 @@ static int free_u_entry(void **entry, void *user)
 	return 0;
 }
 
-void *FN(UNION,free)(__isl_take UNION *u)
+__isl_null UNION *FN(UNION,free)(__isl_take UNION *u)
 {
 	if (!u)
 		return NULL;
@@ -673,13 +672,13 @@ __isl_give UNION *FN(UNION,gist)(__isl_take UNION *u,
 }
 
 #ifndef NO_EVAL
-__isl_give isl_qpolynomial *FN(UNION,eval)(__isl_take UNION *u,
+__isl_give isl_val *FN(UNION,eval)(__isl_take UNION *u,
 	__isl_take isl_point *pnt)
 {
 	uint32_t hash;
 	struct isl_hash_table_entry *entry;
 	isl_space *space;
-	isl_qpolynomial *qp;
+	isl_val *v;
 
 	if (!u || !pnt)
 		goto error;
@@ -694,13 +693,13 @@ __isl_give isl_qpolynomial *FN(UNION,eval)(__isl_take UNION *u,
 				    hash, &has_dim, space, 0);
 	isl_space_free(space);
 	if (!entry) {
-		qp = isl_qpolynomial_zero_on_domain(isl_space_copy(pnt->dim));
+		v = isl_val_zero(isl_point_get_ctx(pnt));
 		isl_point_free(pnt);
 	} else {
-		qp = FN(PART,eval)(FN(PART,copy)(entry->data), pnt);
+		v = FN(PART,eval)(FN(PART,copy)(entry->data), pnt);
 	}
 	FN(UNION,free)(u);
-	return qp;
+	return v;
 error:
 	FN(UNION,free)(u);
 	isl_point_free(pnt);
