@@ -1,20 +1,20 @@
 {
   write_shared_A(3, ((t1 + 3) % 4) + 1, ((t2 + 31) % 32) + 1);
   if (t1 % 3 == 0 && t2 >= 1 && t2 <= 2)
-    write_shared_A(3, (-t1 + 12) / 3, t2 + 32);
+    write_shared_A(3, (-t1 / 3) + 4, t2 + 32);
   {
     int c3 = ((t1 + 3) % 4) + 1 >= t2 && t2 >= 2 && t2 <= 33 ? t2 + 32 : ((t2 + 30) % 32) + 2;
-    if (c3 == t2 + 32 || (c3 == t2 && ((-t1 + 8) % 4) + t2 >= ((t2 + 1) % 2) + 5))
+    if (c3 == t2 + 32 || (c3 == t2 && t2 >= ((t1 + 3) % 4) + ((t2 + 1) % 2) + 2))
       write_shared_A(3, ((t1 + 3) % 4) + 5, c3);
   }
   if (t2 >= t1 + 1 && t2 <= 4 && t1 >= 1)
     write_shared_A(3, t1 + 4, t2 + 32);
   write_shared_A(4, ((t1 + 3) % 4) + 1, ((t2 + 31) % 32) + 1);
   if (t1 % 3 == 0 && t2 >= 1 && t2 <= 2)
-    write_shared_A(4, (-t1 + 12) / 3, t2 + 32);
+    write_shared_A(4, (-t1 / 3) + 4, t2 + 32);
   {
     int c3 = ((t1 + 3) % 4) + 1 >= t2 && t2 >= 2 && t2 <= 33 ? t2 + 32 : ((t2 + 30) % 32) + 2;
-    if (c3 == t2 + 32 || (c3 == t2 && ((-t1 + 8) % 4) + t2 >= ((t2 + 1) % 2) + 5))
+    if (c3 == t2 + 32 || (c3 == t2 && t2 >= ((t1 + 3) % 4) + ((t2 + 1) % 2) + 2))
       write_shared_A(4, ((t1 + 3) % 4) + 5, c3);
   }
   if (t2 >= t1 + 1 && t2 <= 4 && t1 >= 1)
